% This is simulation code provided to help with the development of
% IEEE 802.3dm.
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the author.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.0, December 16th, 2024
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
function state = afe(state,rx_in,tx_in)
%  Analog Front End
	if(nargin<1)
		clear state;
		state.rx_gain = 1;
		state.noise = Noise();
		state.dac = Hold();
		state.tx_filter = Filter();
		state.rx_filter = Filter();
		state.adc_in = 0;
		state.adc_sampling = Resample();
		state.adc = Quantize();
		state.rx_out = 0;
		state.tx_out = 0;
	else
		state.dac = Hold(state.dac,tx_in);
		state.tx_filter = Filter(state.tx_filter,state.dac.out);
		state.rx_filter = Filter(state.rx_filter,rx_in);
		state.noise = Noise(state.noise,state.rx_filter.out);
		state.add_noise = Add(state.rx_filter.out,state.noise.out);
		state.adc_in = Mult(state.add_noise,state.rx_gain);
		state.adc_sampling = Resample(state.adc_sampling,state.adc_in);
		state.adc = Quantize(state.adc,state.adc_sampling.out);
		state.rx_out = state.adc.out;
		state.tx_out = state.tx_filter.out;
	end
end
