% This is simulation code provided to help with the development of
% IEEE 802.3dm.
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the author.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.0, December 16th, 2024
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
function state = pcb(state,rx_in,tx_in)
%  PCB and Packaging
	if(nargin<1)
		clear state;
		state.noise = Noise();
		state.tx_filter = Filter();
		state.rx_filter = Filter();
		state.echo_filter = Filter();
		state.rx_out = 0;
		state.tx_out = 0;
		state.echo_filter.b = 0;
	else
		state.noise = Noise(state.noise,rx_in);
		signal_in = Add(rx_in,state.noise.out);
		state.tx_filter = Filter(state.tx_filter,tx_in);
		state.rx_filter = Filter(state.rx_filter,signal_in);
		state.echo_filter = Filter(state.echo_filter,tx_in);
		state.rx_out = Add(state.rx_filter.out,state.echo_filter.out);
		state.tx_out = state.tx_filter.out;
	end
end

