% This is simulation code provided to help with the development of
% IEEE 802.3dm.
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the author.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.0, December 16th, 2024
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
function state = pma(state,rx_in,tx_in)
%  PMA digital processing
	if(nargin<1)
		clear state;
		state.mapper = Hold();
		state.ffe = Filter();
		state.dfe = Filter();
		state.slicer = Quantize();
		state.ffe_oversampling = 1;
		state.rx_out = 0;
		state.tx_out = 0;
	else
		state.mapper = Hold(state.mapper,tx_in);
		state.ffe = Filter(state.ffe,rx_in);
		ffe_out = state.ffe.out(1:state.ffe_oversampling:end);
		N = length(ffe_out); 
		state.rx_out = zeros(1,N);
		for n = 1:N,
			slicer_in = Add(ffe_out(n),state.dfe.out);
			state.slicer = Quantize(state.slicer,slicer_in);
			state.dfe = Filter(state.dfe,state.slicer.out);
			state.rx_out(n) = state.slicer.out;
		end
		state.tx_out = state.mapper.out;
	end
end
